/*
 * The contents of this file are licenced. You may obtain a copy of
 * the license at https://github.com/thsmi/sieve/ or request it via
 * email from the author.
 *
 * Do not remove or change this comment.
 *
 * The initial author of the code is:
 *   Thomas Schmid <schmid-thomas@gmx.net>
 *
 */

/* global window */

(function () {

  "use strict";

  /* global SieveGrammar */

  if (!SieveGrammar)
    throw new Error("Could not register Mailbox");

  // fileinto [:create] <mailbox: string>
  let create = {
    node: "action/fileinto/create",
    type: "action/fileinto/",

    requires: "mailbox",

    token: ":create"
  };

  SieveGrammar.addTag(create);

  let fileinto = {
    extends: "action/fileinto",

    properties: [{
      id: "tags",
      optional: true,

      elements: [{
        id: "create",
        type: "action/fileinto/create",
        requires: "mailbox"
      }]
    }]
  };

  SieveGrammar.extendAction(fileinto);


  // mailboxexists <mailbox-names: string-list>
  let mailboxexists = {
    node: "test/mailboxexists",
    type: "test",

    requires: "mailbox",

    token: "mailboxexists",

    properties: [{
      id: "parameters",

      elements: [{
        id: "mailboxes",
        type: "stringlist",
        value: '""'
      }]
    }]
  };

  SieveGrammar.addTest(mailboxexists);

  // metadataexists <mailbox: string> <annotation-names: string-list>
  let metadataexists = {
    node: "test/metadataexists",
    type: "test",

    requires: "mboxmetadata",

    token: "metadataexists",

    properties: [{
      id: "parameters",

      elements: [{
        id: "mailbox",
        type: "string",
        value: '""'
      }, {
        id: "annotations",
        type: "stringlist",
        value: '""'
      }]
    }]
  };

  SieveGrammar.addTest(metadataexists);

  // metadata [MATCH-TYPE] [COMPARATOR]
  //         <mailbox: string>
  //         <annotation-name: string> <key-list: string-list>

  /**
   * Retrieves the value of the mailbox annotation "annotation-name" for mailbox
   * "mailbox#". The retrived value is compared against the key-list.
   *
   * The test returns true if the annotation exits and its value matches and of
   * the keys.
   *
   * The default matchtype is :is and the default comparator is "i;ascii-casemap"
   */

  let metadata = {
    node: "test/metadata",
    type: "test",

    requires: "mboxmetadata",

    token: "metadata",

    properties: [{
      id: "tags",
      optional: true,

      elements: [{
        id: "match-type",
        type: "match-type"
      }, {
        id: "comparator",
        type: "comparator"
      }]
    }, {
      id: "parameters",

      elements: [{
        id: "mailbox",
        type: "string",

        value: "\"\""
      }, {
        id: "annotation",
        type: "string",

        value: "\"\""
      }, {
        id: "keys",
        type: "stringlist",

        value: "\"\""
      }]
    }]

  };

  SieveGrammar.addTest(metadata);

  // servermetadataexists <annotation-names: string-list>
  let servermetadataexists = {
    node: "test/servermetadataexists",
    type: "test",

    requires: "servermetadata",

    token: "servermetadataexists",

    properties: [{
      id: "parameters",

      elements: [{
        id: "annotations",
        type: "stringlist",

        value: '""'
      }]
    }]
  };

  SieveGrammar.addTest(servermetadataexists);

  // servermetadata [MATCH-TYPE] [COMPARATOR] <annotation-name: string> <key-list: string-list>
  let servermetadata = {
    node: "test/servermetadata",
    type: "test",

    requires: "servermetadata",

    token: "servermetadata",

    properties: [{
      id: "tags",
      optional: true,

      elements: [{
        id: "match-type",
        type: "match-type"
      }, {
        id: "comparator",
        type: "comparator"
      }]
    }, {
      id: "parameters",

      elements: [{
        id: "annotation",
        type: "string",

        value: '""'
      }, {
        id: "keys",
        type: "stringlist",

        value: '""'
      }]
    }]
  };

  SieveGrammar.addTest(servermetadata);

})(window);
